@@ -25,6 +25,8 @@
 static void      gail_notebook_page_class_init      (GailNotebookPageClass     *klass);
 
 static void                  gail_notebook_page_finalize       (GObject   *object);
+static gint                  gail_notebook_page_label_map_gtk  (GtkWidget *widget,
+                                                                gpointer  data);
 
 static G_CONST_RETURN gchar* gail_notebook_page_get_name       (AtkObject *accessible);
 static AtkObject*            gail_notebook_page_get_parent     (AtkObject *accessible);
@@ -198,6 +200,7 @@ gail_notebook_page_new (GtkNotebook *notebook,
   page->notebook = notebook;
   g_object_add_weak_pointer (G_OBJECT (page->notebook), (gpointer *)&page->notebook);
   page->index = pagenum;
+  page->textutil = NULL;
   
   atk_object = ATK_OBJECT (page);
   atk_object->role = ATK_ROLE_PAGE_TAB;
@@ -212,24 +215,44 @@ gail_notebook_page_new (GtkNotebook *notebook,
   label = get_label_from_notebook_page (page);
   if (GTK_IS_LABEL (label))
     {
-      gail_notebook_page_init_textutil (page, label);
+      if (GTK_WIDGET_MAPPED (label))
+        gail_notebook_page_init_textutil (page, label);
+      else
+        g_signal_connect (label,
+                          "map",
+                          G_CALLBACK (gail_notebook_page_label_map_gtk),
+                          page);
     }
+
   return atk_object;
 }
 
+static gint
+gail_notebook_page_label_map_gtk (GtkWidget *widget,
+                                  gpointer data)
+{
+  GailNotebookPage *page;
+
+  page = GAIL_NOTEBOOK_PAGE (data);
+  gail_notebook_page_init_textutil (page, widget);
+}
+
 static void
 gail_notebook_page_init_textutil (GailNotebookPage *page,
                                   GtkWidget        *label)
 {
   const gchar *label_text;
 
-  page->textutil = gail_text_util_new ();
+  if (page->textutil == NULL)
+    {
+      page->textutil = gail_text_util_new ();
+      g_signal_connect (label,
+                        "notify",
+                        (GCallback) gail_notebook_page_notify,
+                        page);     
+    }
   label_text = gtk_label_get_text (GTK_LABEL (label));
   gail_text_util_text_setup (page->textutil, label_text);
-  g_signal_connect (label,
-                    "notify",
-                    (GCallback) gail_notebook_page_notify,
-                    page);     
 }
 
 static gint
